package com.tools;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

/**
 * 类型的属性字段结构体
 * <br>
 * author 失足程序员<br>
 * blog http://www.cnblogs.com/shizuchengxuyuan/<br>
 * mail 492794628@qq.com<br>
 * phone 13882122019<br>
 */
public class FieldStruc {

    /**
     *
     */
    private final HashMap<String, Field> mapByName = new HashMap<>();
    /**
     *
     */
    private final HashMap<Integer, HashSet<Field>> mapfByNumber = new HashMap<>();

    public FieldStruc() {
    }

    public FieldStruc(Class<?> clazz) {
        actionClass(clazz);
    }

    /**
     * 如果 注解{@link FieldAnn alligator()} true 表示字段不参与计算忽律缓存
     *
     * @param clazz
     */
    protected void actionClass(Class<?> clazz) {
        Field[] declaredFields = clazz.getDeclaredFields();
        for (Field field : declaredFields) {
            if (Modifier.isStatic(field.getModifiers())
                    || Modifier.isFinal(field.getModifiers())) {
                continue;
            }

            field.setAccessible(true);

            FieldAnn annotation = field.getAnnotation(FieldAnn.class);
            if (annotation != null) {
                if (annotation.alligator()) {
                    /*忽律字段*/
                    continue;
                }
                if (annotation.fieldNumber() > 0) {
                    if (!mapfByNumber.containsKey(annotation.fieldNumber())) {
                        mapfByNumber.put(annotation.fieldNumber(), new HashSet<>());
                    }
                    mapfByNumber.get(annotation.fieldNumber()).add(field);
                }
            }

            mapByName.put(field.getName(), field);
        }
        Class<?> superclass = clazz.getSuperclass();
        if (superclass != null) {
            actionClass(superclass);
        }
    }

    /**
     * 字段名称
     *
     * @return
     */
    public HashMap<String, Field> getMapByName() {
        return mapByName;
    }

    /**
     * 字段数值类型 key 值来源 {@link FieldAnn}
     *
     * @return
     */
    public HashMap<Integer, HashSet<Field>> getMapfByNumber() {
        return mapfByNumber;
    }

    /**
     * 设置属性，累加属性
     *
     * @param fieldTypeId
     * @param source
     * @param value
     */
    public void sumFieldValue(int fieldTypeId, Object source, Object value) {
        HashSet<Field> attset = getMapfByNumber().get(fieldTypeId);
        if (attset != null) {
            for (Field field : attset) {
                FieldUtil.sumFieldValue(source, field, value);
            }
        } else {
            System.out.println("属性ID不存在：" + fieldTypeId);
        }
    }

    /**
     * 设置属性，累加属性
     *
     * @param fieldName
     * @param source
     * @param value
     */
    public void sumFieldValue(String fieldName, Object source, Object value) {
        Field field = getMapByName().get(fieldName);
        if (field != null) {
            FieldUtil.sumFieldValue(source, field, value);
        } else {
            System.out.println("属性名字不存在：" + fieldName);
        }
    }

    /**
     * 属性字段做减法操作
     * <br>非数值类型直接替换
     *
     * @param fieldTypeId
     * @param source
     * @param value
     */
    public void subtractFieldValue(int fieldTypeId, Object source, Object value) {
        HashSet<Field> attset = getMapfByNumber().get(fieldTypeId);
        if (attset != null) {
            for (Field field : attset) {
                FieldUtil.subtractFieldValue(source, field, value);
            }
        } else {
            System.out.println("属性ID不存在：" + fieldTypeId);
        }
    }

    /**
     * 属性字段做减法操作
     * <br>非数值类型直接替换
     *
     * @param fieldName
     * @param source
     * @param value
     */
    public void subtractFieldValue(String fieldName, Object source, Object value) {
        Field field = getMapByName().get(fieldName);
        if (field != null) {
            FieldUtil.subtractFieldValue(source, field, value);
        } else {
            System.out.println("属性名字不存在：" + fieldName);
        }
    }

    /**
     * 乘以对于的值，如果是非数值类型，忽律
     *
     * @param fieldTypeId
     * @param source
     * @param value
     */
    public void multiplyFieldValue(int fieldTypeId, Object source, Object value) {
        HashSet<Field> attset = getMapfByNumber().get(fieldTypeId);
        if (attset != null) {
            for (Field field : attset) {
                FieldUtil.multiplyFieldValue(source, field, value);
            }
        } else {
            System.out.println("属性ID不存在：" + fieldTypeId);
        }
    }

    /**
     * 乘以对于的值，如果是非数值类型，忽律
     *
     * @param fieldName
     * @param source
     * @param value
     */
    public void multiplyFieldValue(String fieldName, Object source, Object value) {
        Field field = getMapByName().get(fieldName);
        if (field != null) {
            FieldUtil.multiplyFieldValue(source, field, value);
        } else {
            System.out.println("属性名字不存在：" + fieldName);
        }
    }

    /**
     * 设置属性
     *
     * @param fieldName
     * @param source
     * @param value
     */
    public void setFieldValue(String fieldName, Object source, Object value) {
        Field field = getMapByName().get(fieldName);
        if (field != null) {
            FieldUtil.setFieldValue(source, field, value);
        } else {
            System.out.println("属性名字不存在：" + fieldName);
        }
    }

    /**
     * 设置属性
     *
     * @param fieldTypeId
     * @param source
     * @param value
     */
    public void setFieldValue(int fieldTypeId, Object source, Object value) {
        HashSet<Field> attset = getMapfByNumber().get(fieldTypeId);
        if (attset != null) {
            for (Field field : attset) {
                FieldUtil.setFieldValue(source, field, value);
            }
        } else {
            System.out.println("属性ID不存在：" + fieldTypeId);
        }
    }

    /**
     * 第一个参数值 加上 第二个参数的属性值
     *
     * @param source1
     * @param source2
     */
    public void sumFieldValue(Object source1, Object source2) {
        HashMap<String, Field> mapByName1 = this.getMapByName();
        for (Map.Entry<String, Field> entry : mapByName1.entrySet()) {
            String key = entry.getKey();
            Field field = entry.getValue();
            /*读取第二个参数的属性值*/
            Object fieldValue = FieldUtil.getFieldValue(source2, field);
            /*追加到抵押给参数的属性值*/
            FieldUtil.sumFieldValue(source1, field, fieldValue);
        }
    }

    /**
     * 第一个参数值 减去 第二个参数的属性值
     *
     * @param source1
     * @param source2
     */
    public void subtractFieldValue(Object source1, Object source2) {
        HashMap<String, Field> mapByName1 = this.getMapByName();
        for (Map.Entry<String, Field> entry : mapByName1.entrySet()) {
            String key = entry.getKey();
            Field field = entry.getValue();
            /*读取第二个参数的属性值*/
            Object fieldValue = FieldUtil.getFieldValue(source2, field);
            /*追加到抵押给参数的属性值*/
            FieldUtil.subtractFieldValue(source1, field, fieldValue);
        }
    }

    /**
     * 乘以对于的值，如果是非数值类型，忽律
     *
     * @param source1
     * @param source2
     */
    public void multiplyFieldValue(Object source1, Object source2) {
        HashMap<String, Field> mapByName1 = this.getMapByName();
        for (Map.Entry<String, Field> entry : mapByName1.entrySet()) {
            String key = entry.getKey();
            Field field = entry.getValue();
            /*读取第二个参数的属性值*/
            Object fieldValue = FieldUtil.getFieldValue(source2, field);
            /*追加到抵押给参数的属性值*/
            FieldUtil.multiplyFieldValue(source1, field, fieldValue);
        }
    }

    /**
     * 乘以对于的值，如果是非数值类型，忽律
     * <br> 按照万分比配置的数值进行 1+(value/10000);
     *
     * @param source1
     * @param source2
     */
    public void multiplyFieldValueByRatio1(Object source1, Object source2) {
        HashMap<String, Field> mapByName1 = this.getMapByName();
        for (Map.Entry<String, Field> entry : mapByName1.entrySet()) {
            String key = entry.getKey();
            Field field = entry.getValue();
            /*读取第二个参数的属性值*/
            Object fieldValue = FieldUtil.getFieldValue(source2, field);
            /*追加到抵押给参数的属性值*/
            FieldUtil.multiplyFieldValueByRatio1(source1, field, fieldValue);
        }
    }

    /**
     * source1 = source2 的字段属性值 * (source3 的字段属性值 / 100)
     *
     * @param source1
     * @param source2
     * @param source3
     */
    public void setFieldValue1(Object source1, Object source2, Object source3) {
        for (Map.Entry<String, Field> entry : mapByName.entrySet()) {
            String key = entry.getKey();
            Field ms = entry.getValue();
            try {
                if (ms.getType().isAssignableFrom(int.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * (Double) FieldUtil.getFieldValue(source3, ms) / 10000D)).intValue());
                } else if (ms.getType().isAssignableFrom(long.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * (Double) FieldUtil.getFieldValue(source3, ms) / 10000D)).longValue());
                } else if (ms.getType().isAssignableFrom(float.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * (Double) FieldUtil.getFieldValue(source3, ms) / 10000D)).floatValue());
                } else if (ms.getType().isAssignableFrom(double.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) FieldUtil.getFieldValue(source2, ms) * (Double) FieldUtil.getFieldValue(source3, ms) / 10000D));
                } else if (ms.getType().isAssignableFrom(byte.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * (Double) FieldUtil.getFieldValue(source3, ms) / 10000D)).byteValue());
                } else if (ms.getType().isAssignableFrom(short.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * (Double) FieldUtil.getFieldValue(source3, ms) / 10000D)).shortValue());
                }
            } catch (Exception ex) {
                System.out.println("addPercent" + ms.getName());
                ex.printStackTrace(System.out);
            }
        }
    }

    /**
     * source1 = source2 的字段属性值 * (source3 的字段属性值 / 100)
     *
     * @param source1
     * @param source2
     * @param source3
     */
    public void setFieldValue2(Object source1, Object source2, Object source3) {
        for (Map.Entry<String, Field> entry : mapByName.entrySet()) {
            Field ms = entry.getValue();
            try {
                if (ms.getType().isAssignableFrom(int.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * (Double) FieldUtil.getFieldValue(source3, ms) / 100D)).intValue());
                } else if (ms.getType().isAssignableFrom(long.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * (Double) FieldUtil.getFieldValue(source3, ms) / 100D)).longValue());
                } else if (ms.getType().isAssignableFrom(float.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * (Double) FieldUtil.getFieldValue(source3, ms) / 100D)).floatValue());
                } else if (ms.getType().isAssignableFrom(double.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) FieldUtil.getFieldValue(source2, ms) * (Double) FieldUtil.getFieldValue(source3, ms) / 10000D));
                } else if (ms.getType().isAssignableFrom(byte.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * (Double) FieldUtil.getFieldValue(source3, ms) / 100D)).byteValue());
                } else if (ms.getType().isAssignableFrom(short.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * (Double) FieldUtil.getFieldValue(source3, ms) / 100D)).shortValue());
                }
            } catch (Exception ex) {
                System.out.println("addPercent" + ms.getName());
                ex.printStackTrace(System.out);
            }
        }
    }

    /**
     *
     * @param source1
     * @param source2
     * @param ratio
     */
    public void setFieldValueByFloat(Object source1, Object source2, float ratio) {
        for (Map.Entry<String, Field> entry : mapByName.entrySet()) {
            Field ms = entry.getValue();
            try {
                if (ms.getType().isAssignableFrom(int.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * ratio)).intValue());
                } else if (ms.getType().isAssignableFrom(long.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * ratio)).longValue());
                } else if (ms.getType().isAssignableFrom(float.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * ratio)).floatValue());
                } else if (ms.getType().isAssignableFrom(double.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) FieldUtil.getFieldValue(source2, ms) * ratio));
                } else if (ms.getType().isAssignableFrom(byte.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * ratio)).byteValue());
                } else if (ms.getType().isAssignableFrom(short.class)) {
                    FieldUtil.setFieldValue(source1, ms, ((Double) ((Double) FieldUtil.getFieldValue(source2, ms) * ratio)).shortValue());
                }
            } catch (Exception ex) {
                System.out.println("addPercent" + ms.getName());
                ex.printStackTrace(System.out);
            }
        }
    }

    /**
     * 乘以对于的值，如果是非数值类型，忽律
     * <br> 按照百分比配置的数值进行 1+(value/100);
     *
     * @param source1
     * @param source2
     */
    public void multiplyFieldValueByRatio2(Object source1, Object source2) {
        HashMap<String, Field> mapByName1 = this.getMapByName();
        for (Map.Entry<String, Field> entry : mapByName1.entrySet()) {
            String key = entry.getKey();
            Field field = entry.getValue();
            /*读取第二个参数的属性值*/
            Object fieldValue = FieldUtil.getFieldValue(source2, field);
            /*追加到抵押给参数的属性值*/
            FieldUtil.multiplyFieldValueByRatio2(source1, field, fieldValue);
        }
    }

    /**
     * 如果有一个属性不为 0 就 返回 false
     *
     * @param source
     * @return
     */
    public boolean isFieldValueZore(Object source) {
        HashMap<String, Field> mapByName1 = this.getMapByName();
        for (Map.Entry<String, Field> entry : mapByName1.entrySet()) {
            String key = entry.getKey();
            Field field = entry.getValue();
            if (FieldUtil.isZore(field, source) != 0) {
                return false;
            }
        }
        return true;
    }

    /**
     * 如果属性小于0.直接设置0;
     *
     * @param source
     */
    public void clearFieldValueZore(Object source) {
        HashMap<String, Field> mapByName1 = this.getMapByName();
        for (Map.Entry<String, Field> entry : mapByName1.entrySet()) {
            String key = entry.getKey();
            Field field = entry.getValue();
            if (FieldUtil.isNumberFrom(field) && FieldUtil.isZore(field, source) < 0) {
                FieldUtil.setFieldValue(source, field, 0);
            }
        }
    }

    /**
     * 所有属性都归0；
     *
     * @param source
     */
    public void clearZore(Object source) {
        HashMap<String, Field> mapByName1 = this.getMapByName();
        for (Map.Entry<String, Field> entry : mapByName1.entrySet()) {
            String key = entry.getKey();
            Field field = entry.getValue();
            if (FieldUtil.isNumberFrom(field)) {
                FieldUtil.setFieldValue(source, field, 0);
            }
        }
    }

    /**
     *
     * @param source1
     * @param ratio
     */
    public void multiplyValue(Object source1, float ratio) {
        HashMap<String, Field> mapByName1 = this.getMapByName();
        for (Map.Entry<String, Field> entry : mapByName1.entrySet()) {
            Field field = entry.getValue();
            /*追加到抵押给参数的属性值*/
            FieldUtil.multiplyValue(source1, field, ratio);
        }
    }

    /**
     *
     * @param source1
     * @param ratio
     */
    public void divideValue(Object source1, float ratio) {
        HashMap<String, Field> mapByName1 = this.getMapByName();
        for (Map.Entry<String, Field> entry : mapByName1.entrySet()) {
            Field field = entry.getValue();
            /*追加到指定给参数的属性值*/
            FieldUtil.divideValue(source1, field, ratio);
        }
    }

}
