package com.tools;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.Map;

/**
 * 属性辅助类
 * <br>
 * author 失足程序员<br>
 * blog http://www.cnblogs.com/shizuchengxuyuan/<br>
 * mail 492794628@qq.com<br>
 * phone 13882122019<br>
 */
public class FieldUtil {

    /**
     *
     * @param <T>
     */
    public interface FieldAction<T extends Number> {

        T action(T t1, T t2);
    }

    /**
     * 获取所有字段，排除静态属性和最终属性字段
     *
     * @param clazz
     * @return
     */
    public static Map<String, Field> getFields(Class<?> clazz) {
        Map<String, Field> mapf = new HashMap<>();
        getFields(clazz, mapf);
        return mapf;
    }

    /**
     * 获取所有字段，排除静态属性和最终属性字段
     *
     * @param clazz
     * @param mapf
     */
    public static void getFields(Class<?> clazz, Map<String, Field> mapf) {
        Field[] declaredFields = clazz.getDeclaredFields();
        for (Field field : declaredFields) {
            if (Modifier.isStatic(field.getModifiers())
                    || Modifier.isFinal(field.getModifiers())) {
                continue;
            }
            field.setAccessible(true);
            mapf.put(field.getName(), field);
        }
        Class<?> superclass = clazz.getSuperclass();
        if (superclass != null) {
            getFields(superclass, mapf);
        }
    }

    /**
     * 获取一个属性字段，包含父类的属性字段查找
     * <br>排除静态属性和最终属性字段
     *
     * @param clazz
     * @param name
     * @return
     */
    public static Field getField(Class<?> clazz, String name) {
        Field[] declaredFields = clazz.getDeclaredFields();
        for (Field field : declaredFields) {
            if (field.getName().equals(name)) {
                if (Modifier.isStatic(field.getModifiers())
                        || Modifier.isFinal(field.getModifiers())) {
                    continue;
                }
                field.setAccessible(true);
                return field;
            }
        }
        Class<?> superclass = clazz.getSuperclass();
        if (superclass != null) {
            return getField(superclass, name);
        }
        return null;
    }

    /**
     * 设置数字类型字段
     *
     * @param source
     * @param field
     * @param value
     */
    public static void setFieldValueByDouble(Object source, Field field, Double value) {
        try {
            field.setAccessible(true);
            if (field.getType() == Integer.class || field.getType() == int.class) {
                setFieldValue(source, field, value.intValue());
            } else if (field.getType() == Float.class || field.getType() == float.class) {
                setFieldValue(source, field, value.floatValue());
            } else if (field.getType() == Double.class || field.getType() == double.class) {
                setFieldValue(source, field, value);
            } else if (field.getType() == Long.class || field.getType() == long.class) {
                setFieldValue(source, field, value.longValue());
            } else if (field.getType() == Byte.class || field.getType() == byte.class) {
                setFieldValue(source, field, value.byteValue());
            } else if (field.getType() == Short.class || field.getType() == short.class) {
                setFieldValue(source, field, value.shortValue());
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 设置属性值
     *
     * @param source
     * @param field
     * @param value
     */
    public static void setFieldValue(Object source, Field field, Object value) {
        try {
            field.setAccessible(true);
            field.set(source, ConvertTypeUtil.changeType(value, field.getType()));
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取属性值
     *
     * @param source
     * @param field
     * @return
     */
    public static <T> T getFieldValue(Object source, Field field) {
        try {
            field.setAccessible(true);
            Object object = field.get(source);
            return (T) ConvertTypeUtil.changeType(object, field.getType());
        } catch (Exception e) {
            System.out.println("获取参数错误");
            e.printStackTrace(System.out);
        }
        return null;
    }

    /**
     * 返回三个数字 -1 表示数字小于零，0 等于0，1 大于零表示
     *
     * @param field
     * @param source
     * @return
     */
    public static int isZore(Field field, Object source) {
        try {
            if (isNumberFrom(field)) {
                double a = field.getDouble(source);
                if (a == 0) {
                    return 0;
                } else if (a > 0) {
                    return 1;
                }
                return -1;
            }
        } catch (Exception e) {
            System.out.println("类型转换错误");
            e.printStackTrace(System.out);
        }
        return 0;
    }

    /**
     * 属性字段做相加操作
     * <br>非基础类型直接替换
     *
     * @param source
     * @param field
     * @param value
     */
    public static void sumFieldValue(Object source, Field field, Object value) {
        if (field == null || value == null) {
            return;
        }
        try {
            if (isNumberFrom(field)) {
                field.setAccessible(true);
                Double a = field.getDouble(source);
                a += ConvertTypeUtil.toDouble(value);
                setFieldValueByDouble(source, field, a);
            }
        } catch (Exception e) {
            System.out.println("类型转换错误");
            e.printStackTrace(System.out);
        }
    }

    /**
     * 是数值类型
     *
     * @param field
     * @return
     */
    public static boolean isNumberFrom(Field field) {
        Class<?> type = field.getType();
        return isNumberFrom(type);
    }

    /**
     * 是数值类型
     *
     * @param field
     * @return
     */
    public static boolean isNumberFrom(Class<?> field) {
        if (field.isAssignableFrom(Number.class)
                || field.isAssignableFrom(byte.class)
                || field.isAssignableFrom(short.class)
                || field.isAssignableFrom(int.class)
                || field.isAssignableFrom(long.class)
                || field.isAssignableFrom(float.class)
                || field.isAssignableFrom(double.class)) {
            return true;
        }
        return false;
    }

    /**
     * 属性字段做减法操作
     * <br>非数值类型直接替换
     *
     * @param source
     * @param field
     * @param value
     */
    public static void subtractFieldValue(Object source, Field field, Object value) {
        if (field == null || value == null) {
            return;
        }
        if (!isNumberFrom(field)) {
            return;
        }
        try {
            field.setAccessible(true);
            Double a = field.getDouble(source);
            a -= ConvertTypeUtil.toDouble(value);
            setFieldValueByDouble(source, field, a);
        } catch (Exception e) {
            System.out.println("类型转换错误");
            e.printStackTrace(System.out);
        }
    }

    /**
     * 乘以对于的值，如果是非数值类型，忽律
     *
     * @param source
     * @param field
     * @param value
     */
    public static void multiplyFieldValue(Object source, Field field, Object value) {
        if (field == null || value == null) {
            return;
        }
        if (!isNumberFrom(field)) {
            return;
        }
        try {
            field.setAccessible(true);
            Double a = field.getDouble(source);
            a *= ConvertTypeUtil.toDouble(value);
            setFieldValueByDouble(source, field, a);
        } catch (Exception e) {
            System.out.println("类型转换错误");
            e.printStackTrace(System.out);
        }
    }

    /**
     * 倍率
     *
     * @param source
     * @param field
     * @param value 按倍率提升
     */
    public static void multiplyValue(Object source, Field field, float value) {
        try {
            if (!isNumberFrom(field)) {
                return;
            }
            field.setAccessible(true);
            Double a = field.getDouble(source);
            a *= value;
            setFieldValueByDouble(source, field, a);
        } catch (Exception e) {
            System.out.println("类型转换错误");
            e.printStackTrace(System.out);
        }
    }

    /**
     * 除，如果是非数值类型，忽律
     *
     * @param source
     * @param field
     * @param value
     */
    public static void divideFieldValue(Object source, Field field, Object value) {
        if (field == null || value == null) {
            return;
        }
        try {
            if (!isNumberFrom(field)) {
                return;
            }
            field.setAccessible(true);
            Double a = field.getDouble(source);
            a /= ConvertTypeUtil.toDouble(value);
            setFieldValueByDouble(source, field, a);
        } catch (Exception e) {
            System.out.println("类型转换错误");
            e.printStackTrace(System.out);
        }
    }

    /**
     * 除
     *
     * @param source
     * @param field
     * @param value
     */
    public static void divideValue(Object source, Field field, float value) {
        if (field == null || value == 0) {
            return;
        }
        try {
            if (!isNumberFrom(field)) {
                return;
            }
            field.setAccessible(true);
            Double a = field.getDouble(source);
            a /= value;
            setFieldValueByDouble(source, field, a);
        } catch (Exception e) {
            System.out.println("类型转换错误");
            e.printStackTrace(System.out);
        }
    }

    /**
     * 乘以对于的值，如果是非数值类型，忽律
     * <br> 按照万分比配置的数值进行 1+(value/10000);
     *
     * @param source
     * @param field
     * @param value
     */
    public static void multiplyFieldValueByRatio1(Object source, Field field, Object value) {
        if (field == null || value == null) {
            return;
        }
        try {
            if (!isNumberFrom(field)) {
                return;
            }
            field.setAccessible(true);
            Double a = field.getDouble(source);
            a *= 1 + ConvertTypeUtil.toDouble(value) / 10000;
            setFieldValueByDouble(source, field, a);
        } catch (Exception e) {
            System.out.println("类型转换错误");
            e.printStackTrace(System.out);
        }
    }

    /**
     * 乘以对于的值，如果是非数值类型，忽律
     * <br> 按照百分比配置的数值进行 1+(value/100);
     *
     * @param source
     * @param field
     * @param value
     */
    public static void multiplyFieldValueByRatio2(Object source, Field field, Object value) {
        if (field == null || value == null) {
            return;
        }
        try {
            if (!isNumberFrom(field)) {
                return;
            }
            field.setAccessible(true);
            Double a = field.getDouble(source);
            a *= 1 + ConvertTypeUtil.toDouble(value) / 100;
            setFieldValueByDouble(source, field, a);
        } catch (Exception e) {
            System.out.println("类型转换错误");
            e.printStackTrace(System.out);
        }
    }

}
